@@ -17,6 +17,7 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
+#include <config-gwenview.h>
 // Qt
 
 // KDE
@@ -24,8 +25,10 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <kactioncollection.h>
 #include <kapplication.h>
 #include <kcmdlineargs.h>
+#include <kdebug.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <kurl.h>
 
 // Local
 #include <lib/imageformats/imageformats.h>
@@ -36,7 +39,7 @@ int main(int argc, char *argv[]) {
 		"gwenview",        /* appname */
 		0,                 /* catalogName */
 		ki18n("Gwenview"), /* programName */
-		"2.1.2");          /* version */
+		"2.2.0");          /* version */
 	aboutData.setShortDescription(ki18n("An Image Viewer"));
 	aboutData.setLicense(KAboutData::License_GPL);
 	aboutData.setCopyrightStatement(ki18n("Copyright 2000-2008 Aurélien Gâteau"));
