@@ -41,7 +41,7 @@ EmptyDocumentImpl::~EmptyDocumentImpl()
 void EmptyDocumentImpl::init() {
 }
 
-bool EmptyDocumentImpl::isMetaDataLoaded() const {
+bool EmptyDocumentImpl::isMetaInfoLoaded() const {
 	return false;
 }
 
@@ -49,12 +49,5 @@ Document::LoadingState EmptyDocumentImpl::loadingState() const {
 	return Document::LoadingFailed;
 }
 
-Document::SaveResult EmptyDocumentImpl::save(const KUrl&, const QByteArray&) {
-	return Document::SR_OtherError;
-}
-
-void EmptyDocumentImpl::setImage(const QImage&) {
-	// Don't do anything for now, but we could imagine switching to loaded impl
-}
 
 } // namespace
