@@ -28,6 +28,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <QDropEvent>
 #include <QPainter>
 #include <QPointer>
+#include <QQueue>
 #include <QTimer>
 
 // KDE
@@ -43,13 +44,24 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 namespace Gwenview {
 
+#undef ENABLE_LOG
+#undef LOG
+//#define ENABLE_LOG
+#ifdef ENABLE_LOG
+#define LOG(x) kDebug() << x
+#else
+#define LOG(x) ;
+#endif
+
 /** How many pixels between items */
 const int SPACING = 11;
 
+/** How many msec to wait before starting to smooth thumbnails */
+const int SMOOTH_DELAY = 500;
 
 static KFileItem fileItemForIndex(const QModelIndex& index) {
 	if (!index.isValid()) {
-		kWarning() << "Invalid index";
+		LOG("Invalid index");
 		return KFileItem();
 	}
 	QVariant data = index.data(KDirModel::FileItemRole);
@@ -63,28 +75,59 @@ static KUrl urlForIndex(const QModelIndex& index) {
 }
 
 struct Thumbnail {
-	QPixmap normalPix;
-	QPixmap largePix;
-	QSize fullSize;
+	Thumbnail(const QPersistentModelIndex& index_)
+	: mIndex(index_)
+	, mRough(true) {}
 
-	inline const QPixmap& pixmapForGroup(ThumbnailGroup::Enum group) const {
-		return group == ThumbnailGroup::Large ? largePix : normalPix;
+	Thumbnail() : mRough(true) {}
+
+	/**
+	 * Init the thumbnail based on a icon
+	 */
+	void initAsIcon(const QPixmap& pix) {
+		mGroupPix = pix;
+		mFullSize = pix.size();
 	}
 
-	inline QPixmap& pixmapForGroup(ThumbnailGroup::Enum group) {
-		return group == ThumbnailGroup::Large ? largePix : normalPix;
+	bool isGroupPixAdaptedForSize(int size) const {
+		if (mGroupPix.isNull()) {
+			return false;
+		}
+		const int groupSize = qMax(mGroupPix.width(), mGroupPix.height());
+		if (groupSize >= size) {
+			return true;
+		}
+
+		// groupSize is less than size, but this may be because the full image
+		// is the same size as groupSize
+		return groupSize == qMax(mFullSize.width(), mFullSize.height());
 	}
+
+	QPersistentModelIndex mIndex;
+	/// The pix loaded from .thumbnails/{large,normal}
+	QPixmap mGroupPix;
+	/// Scaled version of mGroupPix, adjusted to ThumbnailView::thumbnailSize
+	QPixmap mAdjustedPix;
+	/// Size of the full image
+	QSize mFullSize;
+	/// Whether mAdjustedPix represents has been scaled using fast or smooth
+	//transformation
+	bool mRough;
 };
 
-typedef QMap<QUrl, Thumbnail> ThumbnailForUrlMap;
+typedef QHash<QUrl, Thumbnail> ThumbnailForUrl;
+typedef QQueue<KUrl> UrlQueue;
 
 struct ThumbnailViewPrivate {
 	ThumbnailView* that;
 	int mThumbnailSize;
 	AbstractThumbnailViewHelper* mThumbnailViewHelper;
-	ThumbnailForUrlMap mThumbnailForUrl;
-	QMap<QUrl, QPersistentModelIndex> mPersistentIndexForUrl;
+	ThumbnailForUrl mThumbnailForUrl;
 	QTimer mScheduledThumbnailGenerationTimer;
+
+	UrlQueue mSmoothThumbnailQueue;
+	QTimer mSmoothThumbnailTimer;
+
 	QPixmap mWaitingThumbnail;
 	QPointer<ThumbnailLoadJob> mThumbnailLoadJob;
 
@@ -92,6 +135,7 @@ struct ThumbnailViewPrivate {
 		if (mThumbnailLoadJob) {
 			mThumbnailLoadJob->removeItems(mThumbnailLoadJob->pendingItems());
 		}
+		mSmoothThumbnailQueue.clear();
 		mScheduledThumbnailGenerationTimer.start();
 	}
 
@@ -102,7 +146,7 @@ struct ThumbnailViewPrivate {
 		QPixmap pix;
 		QSize fullSize;
 		mThumbnailViewHelper->thumbnailForDocument(url, group, &pix, &fullSize);
-		mPersistentIndexForUrl[url] = QPersistentModelIndex(index);
+		mThumbnailForUrl[url] = Thumbnail(QPersistentModelIndex(index));
 		that->setThumbnail(item, pix, fullSize);
 	}
 
@@ -112,6 +156,8 @@ struct ThumbnailViewPrivate {
 			mThumbnailLoadJob = new ThumbnailLoadJob(list, group);
 			QObject::connect(mThumbnailLoadJob, SIGNAL(thumbnailLoaded(const KFileItem&, const QPixmap&, const QSize&)),
 				that, SLOT(setThumbnail(const KFileItem&, const QPixmap&, const QSize&)));
+			QObject::connect(mThumbnailLoadJob, SIGNAL(thumbnailLoadingFailed(const KFileItem&)),
+				that, SLOT(setBrokenThumbnail(const KFileItem&)));
 			mThumbnailLoadJob->start();
 		} else {
 			mThumbnailLoadJob->setThumbnailGroup(group);
@@ -120,6 +166,19 @@ struct ThumbnailViewPrivate {
 			}
 		}
 	}
+
+	void roughAdjustThumbnail(Thumbnail* thumbnail) {
+		const QPixmap& mGroupPix = thumbnail->mGroupPix;
+		const int groupSize = qMax(mGroupPix.width(), mGroupPix.height());
+		const int fullSize = qMax(thumbnail->mFullSize.width(), thumbnail->mFullSize.height());
+		if (fullSize == groupSize && groupSize <= mThumbnailSize) {
+			thumbnail->mAdjustedPix = mGroupPix;
+			thumbnail->mRough = false;
+		} else {
+			thumbnail->mAdjustedPix = mGroupPix.scaled(mThumbnailSize, mThumbnailSize, Qt::KeepAspectRatio);
+			thumbnail->mRough = true;
+		}
+	}
 };
 
 
@@ -155,6 +214,10 @@ ThumbnailView::ThumbnailView(QWidget* parent)
 	connect(&d->mScheduledThumbnailGenerationTimer, SIGNAL(timeout()),
 		SLOT(generateThumbnailsForVisibleItems()) );
 
+	d->mSmoothThumbnailTimer.setSingleShot(true);
+	connect(&d->mSmoothThumbnailTimer, SIGNAL(timeout()),
+		SLOT(smoothNextThumbnail()) );
+
 	setContextMenuPolicy(Qt::CustomContextMenu);
 	connect(this, SIGNAL(customContextMenuRequested(const QPoint&)),
 		SLOT(showContextMenu()) );
@@ -198,6 +261,18 @@ void ThumbnailView::setThumbnailSize(int value) {
 		d->mWaitingThumbnail = pix;
 	}
 
+	// Stop smoothing
+	d->mSmoothThumbnailTimer.stop();
+	d->mSmoothThumbnailQueue.clear();
+
+	// Clear adjustedPixes
+	ThumbnailForUrl::iterator
+		it = d->mThumbnailForUrl.begin(),
+		end = d->mThumbnailForUrl.end();
+	for (; it!=end; ++it) {
+		it.value().mAdjustedPix = QPixmap();
+	}
+
 	thumbnailSizeChanged(value);
 	setSpacing(SPACING);
 	d->scheduleThumbnailGenerationForVisibleItems();
@@ -221,13 +296,10 @@ AbstractThumbnailViewHelper* ThumbnailView::thumbnailViewHelper() const {
 void ThumbnailView::rowsAboutToBeRemoved(const QModelIndex& parent, int start, int end) {
 	QListView::rowsAboutToBeRemoved(parent, start, end);
 
+	// Remove references to removed items
 	KFileItemList itemList;
 	for (int pos=start; pos<=end; ++pos) {
 		QModelIndex index = model()->index(pos, 0, parent);
-		if (!index.isValid()) {
-			kDebug() << "Skipping invalid index" << pos;
-			continue;
-		}
 		KFileItem item = fileItemForIndex(index);
 		if (item.isNull()) {
 			kDebug() << "Skipping invalid item!" << index.data().toString();
@@ -236,7 +308,7 @@ void ThumbnailView::rowsAboutToBeRemoved(const QModelIndex& parent, int start, i
 
 		QUrl url = item.url();
 		d->mThumbnailForUrl.remove(url);
-		d->mPersistentIndexForUrl.remove(url);
+		d->mSmoothThumbnailQueue.removeAll(url);
 
 		itemList.append(item);
 	}
@@ -244,6 +316,22 @@ void ThumbnailView::rowsAboutToBeRemoved(const QModelIndex& parent, int start, i
 	if (d->mThumbnailLoadJob) {
 		d->mThumbnailLoadJob->removeItems(itemList);
 	}
+
+	// Update current index if it is among the deleted rows
+	const int row = currentIndex().row();
+	if (start <= row && row <= end) {
+		QModelIndex index;
+		if (end < model()->rowCount() - 1) {
+			index = model()->index(end + 1, 0);
+		} else if (start > 0) {
+			index = model()->index(start - 1, 0);
+		}
+		setCurrentIndex(index);
+	}
+
+	// Removing rows might make new images visible, make sure their thumbnail
+	// is generated
+	d->mScheduledThumbnailGenerationTimer.start();
 }
 
 
@@ -266,74 +354,65 @@ void ThumbnailView::emitIndexActivatedIfNoModifiers(const QModelIndex& index) {
 
 
 void ThumbnailView::setThumbnail(const KFileItem& item, const QPixmap& pixmap, const QSize& size) {
-	QUrl url = item.url();
-	QPersistentModelIndex persistentIndex = d->mPersistentIndexForUrl[url];
-	if (!persistentIndex.isValid()) {
+	ThumbnailForUrl::iterator it = d->mThumbnailForUrl.find(item.url());
+	if (it == d->mThumbnailForUrl.end()) {
 		return;
 	}
+	Thumbnail& thumbnail = it.value();
+	thumbnail.mGroupPix = pixmap;
+	thumbnail.mAdjustedPix = QPixmap();
+	thumbnail.mFullSize = size;
 
-	ThumbnailGroup::Enum group = ThumbnailGroup::fromPixelSize(d->mThumbnailSize);
-	Thumbnail& thumbnail = d->mThumbnailForUrl[url];
-	thumbnail.pixmapForGroup(group) = pixmap;
-	thumbnail.fullSize = size;
+	update(thumbnail.mIndex);
+}
 
-	QRect rect = visualRect(persistentIndex);
-	update(rect);
-	viewport()->update(rect);
+
+void ThumbnailView::setBrokenThumbnail(const KFileItem& item) {
+	ThumbnailForUrl::iterator it = d->mThumbnailForUrl.find(item.url());
+	if (it == d->mThumbnailForUrl.end()) {
+		return;
+	}
+	Thumbnail& thumbnail = it.value();
+	thumbnail.initAsIcon(DesktopIcon("image-missing", 48));
+	update(thumbnail.mIndex);
 }
 
 
 QPixmap ThumbnailView::thumbnailForIndex(const QModelIndex& index) {
 	KFileItem item = fileItemForIndex(index);
 	if (item.isNull()) {
-		kWarning() << "Invalid item";
+		LOG("Invalid item");
 		return QPixmap();
 	}
 	KUrl url = item.url();
 
-	QPixmap pix;
-
-	ThumbnailGroup::Enum group = ThumbnailGroup::fromPixelSize(d->mThumbnailSize);
-	ThumbnailForUrlMap::Iterator it = d->mThumbnailForUrl.find(url);
-	if (it != d->mThumbnailForUrl.end()) {
-		Thumbnail& thumbnail = it.value();
-		pix = thumbnail.pixmapForGroup(group);
-		if (pix.isNull() && thumbnail.fullSize.isValid()) {
-			int maxFullSize = qMax(thumbnail.fullSize.width(), thumbnail.fullSize.height());
-			if (group == ThumbnailGroup::Large && !thumbnail.normalPix.isNull()) {
-				// Use an up-sampled version of the normal size thumbnail
-				int thumbnailSize = qMin(d->mThumbnailSize, maxFullSize);
-				pix = thumbnail.normalPix.scaled(thumbnailSize, thumbnailSize, Qt::KeepAspectRatio);
-			} else if (group == ThumbnailGroup::Normal && !thumbnail.largePix.isNull()) {
-				// Generate the normal version from the large version and use
-				// it
-				int thumbnailSize = qMin(ThumbnailGroup::pixelSize(ThumbnailGroup::Normal), maxFullSize);
-				pix = thumbnail.largePix.scaled(thumbnailSize, thumbnailSize, Qt::KeepAspectRatio);
-				thumbnail.normalPix = pix;
-			}
-		}
-	}
-
-	if (pix.isNull()) {
+	ThumbnailForUrl::Iterator it = d->mThumbnailForUrl.find(url);
+	if (it == d->mThumbnailForUrl.end()) {
 		if (ArchiveUtils::fileItemIsDirOrArchive(item)) {
-			pix = item.pixmap(128);
-			// Fill both sizes, because we don't want to show an up-sampled
-			// version of the normal icons.
-			Thumbnail& thumbnail = d->mThumbnailForUrl[url];
-			thumbnail.normalPix = pix;
-			thumbnail.largePix = pix;
-			thumbnail.fullSize = QSize(128, 128);
+			Thumbnail thumbnail = Thumbnail(QPersistentModelIndex(index));
+			thumbnail.initAsIcon(item.pixmap(128));
+			it = d->mThumbnailForUrl.insert(url, thumbnail);
 		} else {
-			pix = d->mWaitingThumbnail;
+			return d->mWaitingThumbnail;
 		}
 	}
 
-	int maxSize = qMax(pix.width(), pix.height());
-	if (maxSize > d->mThumbnailSize) {
-		return pix.scaled(d->mThumbnailSize, d->mThumbnailSize, Qt::KeepAspectRatio);
-	} else {
-		return pix;
+	Thumbnail& thumbnail = it.value();
+	if (thumbnail.mGroupPix.isNull()) {
+		return d->mWaitingThumbnail;
 	}
+
+	// Adjust thumbnail
+	if (thumbnail.mAdjustedPix.isNull()) {
+		d->roughAdjustThumbnail(&thumbnail);
+	}
+	if (thumbnail.mRough && !d->mSmoothThumbnailQueue.contains(url)) {
+		d->mSmoothThumbnailQueue.enqueue(url);
+		if (!d->mSmoothThumbnailTimer.isActive()) {
+			d->mSmoothThumbnailTimer.start(SMOOTH_DELAY);
+		}
+	}
+	return thumbnail.mAdjustedPix;
 }
 
 
@@ -395,6 +474,21 @@ void ThumbnailView::resizeEvent(QResizeEvent* event) {
 }
 
 
+void ThumbnailView::showEvent(QShowEvent* event) {
+	QListView::showEvent(event);
+	d->scheduleThumbnailGenerationForVisibleItems();
+	QTimer::singleShot(0, this, SLOT(scrollToSelectedIndex()));
+}
+
+
+void ThumbnailView::scrollToSelectedIndex() {
+	QModelIndexList list = selectedIndexes();
+	if (list.count() >= 1) {
+		scrollTo(list.first(), PositionAtCenter);
+	}
+}
+
+
 void ThumbnailView::selectionChanged(const QItemSelection& selected, const QItemSelection& deselected) {
 	QListView::selectionChanged(selected, deselected);
 	emit selectionChangedSignal(selected, deselected);
@@ -411,9 +505,16 @@ void ThumbnailView::generateThumbnailsForVisibleItems() {
 	if (!isVisible()) {
 		return;
 	}
-	ThumbnailGroup::Enum group = ThumbnailGroup::fromPixelSize(d->mThumbnailSize);
 	KFileItemList list;
-	QRect viewportRect = viewport()->rect();
+	QRect visibleRect = viewport()->rect();
+	// Adjust visibleRect so that next invisible rows of thumbnails
+	// get generated too
+	if (isWrapping()) {
+		visibleRect.adjust(0, 0, 0, d->mThumbnailSize * 2);
+	} else {
+		visibleRect.adjust(0, 0, visibleRect.width() / 2, 0);
+	}
+
 	for (int row=0; row < model()->rowCount(); ++row) {
 		QModelIndex index = model()->index(row, 0);
 		KFileItem item = fileItemForIndex(index);
@@ -421,7 +522,7 @@ void ThumbnailView::generateThumbnailsForVisibleItems() {
 
 		// Filter out invisible items
 		QRect rect = visualRect(index);
-		if (!viewportRect.intersects(rect)) {
+		if (!visibleRect.intersects(rect)) {
 			continue;
 		}
 
@@ -438,15 +539,15 @@ void ThumbnailView::generateThumbnailsForVisibleItems() {
 		}
 
 		// Filter out items which already have a thumbnail
-		ThumbnailForUrlMap::ConstIterator it = d->mThumbnailForUrl.find(url);
-		if (it != d->mThumbnailForUrl.constEnd() && !it.value().pixmapForGroup(group).isNull()) {
+		ThumbnailForUrl::ConstIterator it = d->mThumbnailForUrl.constFind(url);
+		if (it != d->mThumbnailForUrl.constEnd() && it.value().isGroupPixAdaptedForSize(d->mThumbnailSize)) {
 			continue;
 		}
 
-		// Add the item to our list and to mPersistentIndexForUrl, so that
+		// Add the item to our list and to mThumbnailForUrl, so that
 		// setThumbnail() can find the item to update
 		list << item;
-		d->mPersistentIndexForUrl[url] = QPersistentModelIndex(index);
+		d->mThumbnailForUrl[url] = Thumbnail(QPersistentModelIndex(index));
 	}
 
 	if (!list.empty()) {
@@ -468,4 +569,38 @@ void ThumbnailView::generateThumbnailForIndex(const QModelIndex& index) {
 }
 
 
+void ThumbnailView::smoothNextThumbnail() {
+	if (d->mSmoothThumbnailQueue.isEmpty()) {
+		return;
+	}
+
+	if (d->mThumbnailLoadJob) {
+		// give mThumbnailLoadJob priority over smoothing
+		d->mSmoothThumbnailTimer.start(SMOOTH_DELAY);
+		return;
+	}
+
+	KUrl url = d->mSmoothThumbnailQueue.dequeue();
+	ThumbnailForUrl::Iterator it = d->mThumbnailForUrl.find(url);
+	if (it == d->mThumbnailForUrl.end()) {
+		kWarning() <<  url << " not in mThumbnailForUrl. This should not happen!";
+		return;
+	}
+
+	Thumbnail& thumbnail = it.value();
+	thumbnail.mAdjustedPix = thumbnail.mGroupPix.scaled(d->mThumbnailSize, d->mThumbnailSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+	thumbnail.mRough = false;
+
+	if (thumbnail.mIndex.isValid()) {
+		update(thumbnail.mIndex);
+	} else {
+		kWarning() << "index for" << url << "is invalid. This should not happen!";
+	}
+
+	if (!d->mSmoothThumbnailQueue.isEmpty()) {
+		d->mSmoothThumbnailTimer.start(0);
+	}
+}
+
+
 } // namespace
