@@ -24,9 +24,9 @@
 #include <string>
 #include <vector>
 
-#include <config.h>
+#include "config-kgamma.h"
 
-#ifdef HAVE_SSTREAM
+#if !defined(HAVE_STRSTREAM_H)
 #include <sstream>
 #else
 #include <strstream.h>
