@@ -1108,6 +1108,7 @@ static const char  *static_camera_list[] =
 "Fuji IS-1",
 "Hasselblad CFV",
 "Hasselblad H3D",
+"Hasselblad V96C",
 "Imacon Ixpress 16-megapixel",
 "Imacon Ixpress 22-megapixel",
 "Imacon Ixpress 39-megapixel",
@@ -1301,7 +1302,7 @@ static const char  *static_camera_list[] =
 "RoverShot 3320af",
 "Samsung GX-1S",
 "Samsung GX-10",
-"Samsung S85",
+"Samsung S85 (hacked)",
 "Sarnoff 4096x5440",
 "Sigma SD9",
 "Sigma SD10",
