@@ -95,7 +95,8 @@ bool CHMGenerator::loadDocument( const QString & fileName, QVector< Okular::Page
     foreach(const LCHMParsedEntry &e, topics)
     {
         QDomElement item = m_docSyn.createElement(e.name);
-        item.setAttribute("ViewportName", e.urls.first());
+        if (!e.urls.isEmpty())
+             item.setAttribute("ViewportName", e.urls.first());
         item.setAttribute("Icon", e.imageid);
         if (e.indent == 0) m_docSyn.appendChild(item);
         else lastIndentElement[e.indent - 1].appendChild(item);
@@ -106,6 +107,9 @@ bool CHMGenerator::loadDocument( const QString & fileName, QVector< Okular::Page
     int pageNum = 0;
     foreach(const LCHMParsedEntry &e, topics)
     {
+        if (e.urls.isEmpty())
+            continue;
+
         const QString &url = e.urls.first();
         int pos = url.indexOf ('#');
         QString tmpUrl = pos == -1 ? url : url.left(pos);
@@ -125,8 +129,8 @@ bool CHMGenerator::loadDocument( const QString & fileName, QVector< Okular::Page
     if (!m_syncGen)
     {
         m_syncGen = new KHTMLPart();
-        connect (m_syncGen,SIGNAL(completed()),this,SLOT(slotCompleted()));
     }
+    disconnect( m_syncGen, 0, this, 0 );
 
     for (int i = 0; i < m_pageUrl.count(); ++i)
     {
@@ -135,6 +139,8 @@ bool CHMGenerator::loadDocument( const QString & fileName, QVector< Okular::Page
             m_syncGen->view()->contentsHeight(), Okular::Rotation0 );
     }
 
+    connect( m_syncGen, SIGNAL( completed() ), this, SLOT( slotCompleted() ) );
+
     return true;
 }
 
@@ -167,7 +173,10 @@ void CHMGenerator::preparePageForSyncOperation( int zoom , const QString & url)
 
     QEventLoop loop;
     connect( m_syncGen, SIGNAL( completed() ), &loop, SLOT( quit() ) );
-    loop.exec();
+    connect( m_syncGen, SIGNAL( canceled( QString ) ), &loop, SLOT( quit() ) );
+    // discard any user input, otherwise it breaks the "synchronicity" of this
+    // function
+    loop.exec( QEventLoop::ExcludeUserInputEvents );
 }
 
 void CHMGenerator::slotCompleted()
